@@ -11,7 +11,7 @@ use File::GlobMapper;
 require Exporter;
 our ($VERSION, @ISA, @EXPORT, %EXPORT_TAGS);
 @ISA = qw(Exporter);
-$VERSION = '2.000_12';
+$VERSION = '2.000_14';
 
 @EXPORT = qw( isaFilehandle isaFilename whatIsInput whatIsOutput 
               isaFileGlobString cleanFileGlobString oneTarget
@@ -108,10 +108,10 @@ sub whatIsInput($;$)
     
     if (defined $got && $got eq 'filename' && defined $_[0] && $_[0] eq '-')
     {
-        use IO::File;
+        #use IO::File;
         $got = 'handle';
-        #$_[0] = \*STDIN;
-        $_[0] = new IO::File("<-");
+        $_[0] = *STDIN;
+        #$_[0] = new IO::File("<-");
     }
 
     return $got;
@@ -124,8 +124,8 @@ sub whatIsOutput($;$)
     if (defined $got && $got eq 'filename' && defined $_[0] && $_[0] eq '-')
     {
         $got = 'handle';
-        #$_[0] = \*STDOUT;
-        $_[0] = new IO::File(">-");
+        $_[0] = *STDOUT;
+        #$_[0] = new IO::File(">-");
     }
     
     return $got;
@@ -276,6 +276,14 @@ sub Validator::new
     {
         $obj->croakError("$reportClass: output filename is undef or null string")
             if ! defined $_[1] || $_[1] eq ''  ;
+
+        if (-e $_[1])
+        {
+            if (-d _ )
+            {
+                return $obj->saveErrorString("output file '$_[1]' is a directory");
+            }
+        }
     }
     
     return $obj ;
@@ -314,7 +322,12 @@ sub Validator::validateInputFilenames
             return $self->saveErrorString("input file '$filename' does not exist");
         }
 
-        if (! -r $filename )
+        if (-d _ )
+        {
+            return $self->saveErrorString("input file '$filename' is a directory");
+        }
+
+        if (! -r _ )
         {
             return $self->saveErrorString("cannot open file '$filename': $!");
         }
@@ -408,7 +421,7 @@ sub createSelfTiedObject
 $EXPORT_TAGS{Parse} = [qw( ParseParameters 
                            Parse_any Parse_unsigned Parse_signed 
                            Parse_boolean Parse_custom Parse_string
-                           Parse_store_ref
+                           Parse_multiple Parse_writable_scalar
                          )
                       ];              
 
@@ -421,7 +434,10 @@ use constant Parse_boolean  => 0x08;
 use constant Parse_string   => 0x10;
 use constant Parse_custom   => 0x12;
 
-use constant Parse_store_ref => 0x100 ;
+#use constant Parse_store_ref        => 0x100 ;
+use constant Parse_multiple         => 0x100 ;
+use constant Parse_writable         => 0x200 ;
+use constant Parse_writable_scalar  => 0x400 | Parse_writable ;
 
 use constant OFF_PARSED     => 0 ;
 use constant OFF_TYPE       => 1 ;
@@ -531,12 +547,16 @@ sub IO::Compress::Base::Parameters::parse
         $key = lc $key;
 
         if ($firstTime || ! $sticky) {
+            $x = [ $x ]
+                if $type & Parse_multiple;
+
             $got->{$key} = [0, $type, $value, $x, $first_only, $sticky] ;
         }
 
         $got->{$key}[OFF_PARSED] = 0 ;
     }
 
+    my %parsed = ();
     for my $i (0.. @entered / 2 - 1) {
         my $key = $entered[2* $i] ;
         my $value = $entered[2* $i+1] ;
@@ -551,12 +571,24 @@ sub IO::Compress::Base::Parameters::parse
                                   ! $got->{$canonkey}[OFF_FIRST_ONLY]  ))
         {
             my $type = $got->{$canonkey}[OFF_TYPE] ;
+            my $parsed = $parsed{$canonkey};
+            ++ $parsed{$canonkey};
+
+            return $self->setError("Muliple instances of '$key' found") 
+                if $parsed && $type & Parse_multiple == 0 ;
+
             my $s ;
             $self->_checkType($key, $value, $type, 1, \$s)
                 or return undef ;
-            #$value = $$value unless $type & Parse_store_ref ;
+
             $value = $$value ;
-            $got->{$canonkey} = [1, $type, $value, $s] ;
+            if ($type & Parse_multiple) {
+                $got->{$canonkey}[OFF_PARSED] = 1;
+                push @{ $got->{$canonkey}[OFF_FIXED] }, $s ;
+            }
+            else {
+                $got->{$canonkey} = [1, $type, $value, $s] ;
+            }
         }
         else
           { push (@Bad, $key) }
@@ -582,15 +614,39 @@ sub IO::Compress::Base::Parameters::_checkType
 
     #local $Carp::CarpLevel = $level ;
     #print "PARSE $type $key $value $validate $sub\n" ;
-    if ( $type & Parse_store_ref)
-    {
-        #$value = $$value
-        #    if ref ${ $value } ;
 
-        $$output = $value ;
+    if ($type & Parse_writable_scalar)
+    {
+        return $self->setError("Parameter '$key' not writable")
+            if $validate &&  readonly $$value ;
+
+        if (ref $$value) 
+        {
+            return $self->setError("Parameter '$key' not a scalar reference")
+                if $validate &&  ref $$value ne 'SCALAR' ;
+
+            $$output = $$value ;
+        }
+        else  
+        {
+            return $self->setError("Parameter '$key' not a scalar")
+                if $validate &&  ref $value ne 'SCALAR' ;
+
+            $$output = $value ;
+        }
+
         return 1;
     }
 
+#    if ($type & Parse_store_ref)
+#    {
+#        #$value = $$value
+#        #    if ref ${ $value } ;
+#
+#        $$output = $value ;
+#        return 1;
+#    }
+
     $value = $$value ;
 
     if ($type & Parse_any)
@@ -697,6 +753,128 @@ sub IO::Compress::Base::Parameters::clone
     return bless $obj, 'IO::Compress::Base::Parameters' ;
 }
 
+package U64;
+
+use constant MAX32 => 0xFFFFFFFF ;
+use constant LOW   => 0 ;
+use constant HIGH  => 1;
+
+sub new
+{
+    my $class = shift ;
+
+    my $high = 0 ;
+    my $low  = 0 ;
+
+    if (@_ == 2) {
+        $high = shift ;
+        $low  = shift ;
+    }
+    elsif (@_ == 1) {
+        $low  = shift ;
+    }
+
+    bless [$low, $high], $class;
+}
+
+sub newUnpack_V64
+{
+    my $string = shift;
+
+    my ($low, $hi) = unpack "V V", $string ;
+    bless [ $low, $hi ], "U64";
+}
+
+sub newUnpack_V32
+{
+    my $string = shift;
+
+    my $low = unpack "V", $string ;
+    bless [ $low, 0 ], "U64";
+}
+
+sub reset
+{
+    my $self = shift;
+    $self->[HIGH] = $self->[LOW] = 0;
+}
+
+sub clone
+{
+    my $self = shift;
+    bless [ @$self ], ref $self ;
+}
+
+sub getHigh
+{
+    my $self = shift;
+    return $self->[HIGH];
+}
+
+sub getLow
+{
+    my $self = shift;
+    return $self->[LOW];
+}
+
+sub get32bit
+{
+    my $self = shift;
+    return $self->[LOW];
+}
+
+sub add
+{
+    my $self = shift;
+    my $value = shift;
+
+    if (ref $value eq 'U64') {
+        $self->[HIGH] += $value->[HIGH] ;
+        $value = $value->[LOW];
+    }
+     
+    my $available = MAX32 - $self->[LOW] ;
+
+    if ($value > $available) {
+       ++ $self->[HIGH] ;
+       $self->[LOW] = $value - $available - 1;
+    }
+    else {
+       $self->[LOW] += $value ;
+    }
+}
+
+sub equal
+{
+    my $self = shift;
+    my $other = shift;
+
+    return $self->[LOW]  == $other->[LOW] &&
+           $self->[HIGH] == $other->[HIGH] ;
+}
+
+sub getPacked_V64
+{
+    my $self = shift;
+
+    return pack "V V", @$self ;
+}
+
+sub getPacked_V32
+{
+    my $self = shift;
+
+    return pack "V", $self->[LOW] ;
+}
+
+sub pack_V64
+{
+    my $low  = shift;
+
+    return pack "V V", $low, 0;
+}
+
+
 package IO::Compress::Base::Common;
 
 1;
