@@ -8,25 +8,12 @@ use IO::Compress::Base::Common qw(createSelfTiedObject);
 
 use IO::Uncompress::Base ;
 
-BEGIN
-{
-   eval { require IO::Uncompress::Adapter::Inflate; import IO::Uncompress::Adapter::Inflate };
-   eval { require IO::Uncompress::Adapter::Bunzip2; import IO::Uncompress::Adapter::Bunzip2 };
-   eval { require IO::Uncompress::Adapter::LZO; import IO::Uncompress::Adapter::LZO };
-
-   eval { require IO::Uncompress::Bunzip2; import IO::Uncompress::Bunzip2 };
-   eval { require IO::Uncompress::UnLzop; import IO::Uncompress::UnLzop };
-   eval { require IO::Uncompress::Gunzip; import IO::Uncompress::Gunzip };
-   eval { require IO::Uncompress::Inflate; import IO::Uncompress::Inflate };
-   eval { require IO::Uncompress::RawInflate; import IO::Uncompress::RawInflate };
-   eval { require IO::Uncompress::Unzip; import IO::Uncompress::Unzip };
-}
 
 require Exporter ;
 
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, $AnyUncompressError);
 
-$VERSION = '2.000_13';
+$VERSION = '2.000_14';
 $AnyUncompressError = '';
 
 @ISA = qw( Exporter IO::Uncompress::Base );
@@ -38,6 +25,22 @@ Exporter::export_ok_tags('all');
 # TODO - allow the user to pick a set of the three formats to allow
 #        or just assume want to auto-detect any of the three formats.
 
+BEGIN
+{
+   eval ' use IO::Uncompress::Adapter::Inflate ';
+   eval ' use IO::Uncompress::Adapter::Bunzip2 ';
+   eval ' use IO::Uncompress::Adapter::LZO ';
+   eval ' use IO::Uncompress::Adapter::Lzf ';
+
+   eval ' use IO::Uncompress::Bunzip2 ';
+   eval ' use IO::Uncompress::UnLzop ';
+   eval ' use IO::Uncompress::Gunzip ';
+   eval ' use IO::Uncompress::Inflate ';
+   eval ' use IO::Uncompress::RawInflate ';
+   eval ' use IO::Uncompress::Unzip ';
+   eval ' use IO::Uncompress::UnLzf ';
+}
+
 sub new
 {
     my $class = shift ;
@@ -53,7 +56,8 @@ sub anyuncompress
 
 sub getExtraParams
 {
-    return ();
+    use IO::Compress::Base::Common qw(:Parse);
+    return ( 'RawInflate' => [1, 1, Parse_boolean,  0] ) ;
 }
 
 sub ckParams
@@ -86,17 +90,20 @@ sub mkUncomp
 
         *$self->{Uncomp} = $obj;
         
-         $magic = $self->ckMagic( qw( RawInflate Inflate Gunzip Unzip ) ); 
+        my @possible = qw( Inflate Gunzip Unzip );
+        unshift @possible, 'RawInflate' 
+            if $got->value('RawInflate');
 
-         if ($magic) {
+        $magic = $self->ckMagic( @possible );
+        
+        if ($magic) {
             *$self->{Info} = $self->readHeader($magic)
                 or return undef ;
 
             return 1;
-         }
+        }
      }
 
-     #foreach my $type ( qw( Bunzip2 UnLzop ) ) {
      if (defined $IO::Uncompress::Bunzip2::VERSION and
          $magic = $self->ckMagic('Bunzip2')) {
         *$self->{Info} = $self->readHeader($magic)
@@ -111,7 +118,8 @@ sub mkUncomp
 
          return 1;
      }
-     elsif (defined $IO::Uncompress::UnLzop::VERSION and
+
+     if (defined $IO::Uncompress::UnLzop::VERSION and
             $magic = $self->ckMagic('UnLzop')) {
 
         *$self->{Info} = $self->readHeader($magic)
@@ -127,6 +135,22 @@ sub mkUncomp
          return 1;
      }
 
+     if (defined $IO::Uncompress::UnLzf::VERSION and
+            $magic = $self->ckMagic('UnLzf')) {
+
+        *$self->{Info} = $self->readHeader($magic)
+            or return undef ;
+
+        my ($obj, $errstr, $errno) = IO::Uncompress::Adapter::Lzf::mkUncompObject();
+
+        return $self->saveErrorString(undef, $errstr, $errno)
+            if ! defined $obj;
+
+        *$self->{Uncomp} = $obj;
+
+         return 1;
+     }
+
      return 0 ;
 }
 
@@ -243,7 +267,7 @@ The formats supported are:
 
 =item RFC 1950
 
-=item RFC 1951
+=item RFC 1951 (optionally)
 
 =item gzip (RFC 1952)
 
@@ -253,6 +277,8 @@ The formats supported are:
 
 =item lzop
 
+=item lzf
+
 =back
 
 The module will auto-detect which, if any, of the supported
@@ -261,7 +287,6 @@ compression formats is being used.
 
 
 
-
 =head1 Functional Interface
 
 A top-level function, C<anyuncompress>, is provided to carry out
@@ -435,6 +460,7 @@ TODO
 
 =item C<< MultiStream => 0|1 >>
 
+
 If the input file/buffer contains multiple compressed data streams, this
 option will uncompress the whole lot as a single data stream.
 
@@ -442,6 +468,35 @@ Defaults to 0.
 
 
 
+
+
+=item C<< TrailingData => $scalar >>
+
+Returns the data, if any, that is present immediately after the compressed
+data stream once uncompression is complete. 
+
+This option can be used when there is useful information immediately
+following the compressed data stream, and you don't know the length of the
+compressed data stream.
+
+If the input is a buffer, C<trailingData> will return everything from the
+end of the compressed data stream to the end of the buffer.
+
+If the input is a filehandle, C<trailingData> will return the data that is
+left in the filehandle input buffer once the end of the compressed data
+stream has been reached. You can then use the filehandle to read the rest
+of the input file. 
+
+Don't bother using C<trailingData> if the input is a filename.
+
+
+
+If you know the length of the compressed data stream before you start
+uncompressing, you can avoid having to use C<trailingData> by setting the
+C<InputLength> option.
+
+
+
 =back
 
 
@@ -655,6 +710,19 @@ The default for this option is off.
 
 
 
+=item C<< RawInflate => 0|1 >>
+
+When auto-detecting the compressed format, try to test for raw-deflate (RFC
+1951) content using the C<IO::Uncompress::RawInflate> module. 
+
+The reason this is not default behaviour is because RFC 1951 content can
+only be detected by attempting to uncompress it. This process is error
+prone and can result is false positives.
+
+Defaults to 0.
+
+
+
 
 
 
@@ -711,10 +779,10 @@ Usage is
 
 Reads a single line. 
 
-This method fully supports the use of of the variable C<$/>
-(or C<$INPUT_RECORD_SEPARATOR> or C<$RS> when C<English> is in use) to
-determine what constitutes an end of line. Both paragraph mode and file
-slurp mode are supported. 
+This method fully supports the use of of the variable C<$/> (or
+C<$INPUT_RECORD_SEPARATOR> or C<$RS> when C<English> is in use) to
+determine what constitutes an end of line. Paragraph mode, record mode and
+file slurp mode are all supported. 
 
 
 =head2 getc
@@ -886,8 +954,8 @@ Usage is
     my $status = $z->nextStream();
 
 Skips to the next compressed data stream in the input file/buffer. If a new
-compressed data stream is found, the eof marker will be cleared, C<$.> will
-be reset to 0.
+compressed data stream is found, the eof marker will be cleared and C<$.>
+will be reset to 0.
 
 Returns 1 if a new stream was found, 0 if none was found, and -1 if an
 error was encountered.
@@ -898,7 +966,30 @@ Usage is
 
     my $data = $z->trailingData();
 
-Returns any data that 
+Returns the data, if any, that is present immediately after the compressed
+data stream once uncompression is complete. It only makes sense to call
+this method once the end of the compressed data stream has been
+encountered.
+
+This option can be used when there is useful information immediately
+following the compressed data stream, and you don't know the length of the
+compressed data stream.
+
+If the input is a buffer, C<trailingData> will return everything from the
+end of the compressed data stream to the end of the buffer.
+
+If the input is a filehandle, C<trailingData> will return the data that is
+left in the filehandle input buffer once the end of the compressed data
+stream has been reached. You can then use the filehandle to read the rest
+of the input file. 
+
+Don't bother using C<trailingData> if the input is a filename.
+
+
+
+If you know the length of the compressed data stream before you start
+uncompressing, you can avoid having to use C<trailingData> by setting the
+C<InputLength> option in the constructor.
 
 =head1 Importing 
 
@@ -922,7 +1013,7 @@ Same as doing this
 
 =head1 SEE ALSO
 
-L<Compress::Zlib>, L<IO::Compress::Gzip>, L<IO::Uncompress::Gunzip>, L<IO::Compress::Deflate>, L<IO::Uncompress::Inflate>, L<IO::Compress::RawDeflate>, L<IO::Uncompress::RawInflate>, L<IO::Compress::Bzip2>, L<IO::Uncompress::Bunzip2>, L<IO::Compress::Lzop>, L<IO::Uncompress::UnLzop>, L<IO::Uncompress::AnyInflate>
+L<Compress::Zlib>, L<IO::Compress::Gzip>, L<IO::Uncompress::Gunzip>, L<IO::Compress::Deflate>, L<IO::Uncompress::Inflate>, L<IO::Compress::RawDeflate>, L<IO::Uncompress::RawInflate>, L<IO::Compress::Bzip2>, L<IO::Uncompress::Bunzip2>, L<IO::Compress::Lzop>, L<IO::Uncompress::UnLzop>, L<IO::Compress::Lzf>, L<IO::Uncompress::UnLzf>, L<IO::Uncompress::AnyInflate>
 
 L<Compress::Zlib::FAQ|Compress::Zlib::FAQ>
 
